import os
import sys
from typing import Optional

from dotenv import load_dotenv
import typer

from lupin_grognard.__init__ import __version__
from lupin_grognard.core.check import CheckMethod, check_commit_with_commitizen
from lupin_grognard.core.git import Git
from .core.tools.utils import create_branch_list

load_dotenv()
GROG_BRANCHES = os.getenv("GROG_BRANCHES")

cli = typer.Typer()


@cli.command()
def version():
    print(f"Version: {__version__}")


@cli.command()
def check_commits(
    message: Optional[str] = typer.Option(
        None, "--message", "-m", help="commit message you want to test"
    ),
    branches_name: Optional[str] = typer.Argument(
        default="master, main, dev, development", envvar="GROG_BRANCHES"
    ),
):
    """
    Run 'git log --first-parent --pretty=oneline -50' command

    Check every commit message with commitizen since the last request to merge one of the branches from branches_name

    - With --message option :
    grog check-commits [--message or -m] "your commit message"

    - With branches_name argument: grog check-commits "branch_1, branch_2..."

    You can set GROG_BRANCHES env var in .env, gitlab, github...
    """

    if message:
        check = check_commit_with_commitizen(
            check_method=CheckMethod.MESSAGE,
            commit_message=message,
        )
        if check is not None:
            print(check)

    else:
        branch_list = create_branch_list(branches_name=branches_name)
        git = Git()
        current_branch = git.get_branch_name()
        commit_list_to_check = git.generate_commits_to_check(branch_list=branch_list)
        number_of_commit = len(commit_list_to_check)
        if number_of_commit == 1:
            if current_branch in branch_list:
                print(f"current_branch = {current_branch}")
                print(f"branch_list = {branch_list}")
                print("0 commit to check")
                sys.exit(0)
            else:
                print(f"current_branch = {current_branch}")
                print(f"branch_list = {branch_list}")
                print("Found 0 commits to check since last merge")
                sys.exit(0)
        else:
            commits_to_check = number_of_commit - 1
            if commits_to_check > 1:
                print(f"Found {commits_to_check} commits to check:")
            else:
                print(f"Found {commits_to_check} commit to check:")
            check = check_commit_with_commitizen(
                check_method=CheckMethod.REV_RANGE,
                commits=commit_list_to_check,
            )
            if check is not None:
                print(check)
                sys.exit(1)
